'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _fs = require('fs');

var _pugError = require('pug-error');

var _pugError2 = _interopRequireDefault(_pugError);

var _scope = require('./scope');

var _scope2 = _interopRequireDefault(_scope);

var _babelTypes = require('./babel-types');

var _babelTypes2 = _interopRequireDefault(_babelTypes);

var _blockKey = require('./block-key');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Context = function () {
  function Context(definesScope, key, parent, file, path) {
    (0, _classCallCheck3.default)(this, Context);
    this._variables = new _map2.default();
    this.variablesToDeclare = [];
    this._nextBlockID = 0;

    if (!definesScope && parent) {
      this.variablesToDeclare = parent.variablesToDeclare;
    }
    this._parent = parent;
    this.key = key;
    this.file = file;
    this.path = path;
  }

  (0, _createClass3.default)(Context, [{
    key: 'error',
    value: function error(code, message) {
      var src = (0, _fs.readFileSync)(this.file.opts.filename, 'utf8');
      return (0, _pugError2.default)(code, message, {
        filename: this.file.opts.filename,
        line: (0, _babelTypes.getCurrentLocation)().start.line - 1,
        src: src
      });
    }
  }, {
    key: 'noKey',
    value: function noKey(fn) {
      var childContext = new Context(false, new _blockKey.BaseKey(), this, this.file, this.path);
      var result = fn(childContext);
      childContext.end();
      return result;
    }
  }, {
    key: 'staticBlock',
    value: function staticBlock(fn) {
      var childContext = new Context(false, new _blockKey.StaticBlock(this.key, this._nextBlockID++), this, this.file, this.path);
      var result = fn(childContext);
      childContext.end();
      return result;
    }
  }, {
    key: 'dynamicBlock',
    value: function dynamicBlock(fn) {
      var childContext = new Context(true, new _blockKey.DynamicBlock(this.key, 'src', 0), this, this.file, this.path);
      var result = fn(childContext);
      childContext.end();
      return { result: result, variables: childContext.variablesToDeclare };
    }
  }, {
    key: 'end',
    value: function end() {
      this.key.end();
    }
  }, {
    key: 'getVariable',
    value: function getVariable(name) {
      var variable = this._variables.get(name);
      if (variable) {
        return variable;
      }
      if (this._parent) {
        return this._parent.getVariable(name);
      }
      // TODO: maybe actually verify existance/non-const in parent scope?
      return null;
    }
  }, {
    key: 'declareVariable',
    value: function declareVariable(kind, name) {
      if (typeof name !== 'string') {
        throw new Error('variables may only be declared with strings');
      }
      var oldVariable = this._variables.get(name);
      if (oldVariable) {
        if (oldVariable.kind !== 'var' || kind !== 'var') {
          var err = this.error('DUPLICATE_VARIABLE', 'Duplicate variable ' + name + '.');
          throw err;
        }
        return oldVariable;
      }
      var variable = {
        kind: kind,
        id: this.generateUidIdentifier(name)
      };
      this.variablesToDeclare.push(variable.id);
      this._variables.set(name, variable);
      return variable;
    }
  }, {
    key: 'generateUidIdentifier',
    value: function generateUidIdentifier(name) {
      return this.path.scope.generateUidIdentifier(name);
    }
  }, {
    key: 'getBaseLine',
    value: function getBaseLine() {
      return this.path.node.loc.start.line;
    }
  }], [{
    key: 'create',
    value: function create(file, path) {
      return new Context(true, new _blockKey.BaseKey(), null, file, path);
    }
  }]);
  return Context;
}();

exports.default = Context;