'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _parseExpression = require('../utils/parse-expression');

var _parseExpression2 = _interopRequireDefault(_parseExpression);

var _babelTypes = require('../babel-types');

var _babelTypes2 = _interopRequireDefault(_babelTypes);

var _visitors = require('../visitors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getChildren(node, context) {
  return context.noKey(function (childContext) {
    return (node.code ? [(0, _visitors.visitJsx)(node.code, childContext)] : []).concat(node.block.nodes.map(function (node) {
      return (0, _visitors.visitJsx)(node, childContext);
    }));
  });
}

function getAttributes(node, context) {
  var classes = [];
  var attrs = node.attrs.map(function (_ref) {
    var name = _ref.name,
        val = _ref.val,
        mustEscape = _ref.mustEscape;

    if (/\.\.\./.test(name) && val === true) {
      return _babelTypes2.default.jSXSpreadAttribute((0, _parseExpression2.default)(name.substr(3), context));
    }
    switch (name) {
      case 'for':
        name = 'htmlFor';
        break;
      case 'maxlength':
        name = 'maxLength';
        break;
      case 'class':
        name = 'className';
        break;
    }
    var expr = (0, _parseExpression2.default)(val === true ? 'true' : val, context);
    if (!mustEscape && (!_babelTypes2.default.isStringLiteral(expr) || /(\<\>\&)/.test(val))) {
      throw new Error('Unescaped attributes are not supported in react-pug');
    }
    if (name === 'className') {
      classes.push(expr);
      return null;
    }
    var jsxValue = _babelTypes2.default.asStringLiteral(expr) || _babelTypes2.default.asJSXElement(expr) || _babelTypes2.default.jSXExpressionContainer(expr);
    if (/\.\.\./.test(name)) {
      throw new Error('spread attributes must not have a value');
    }
    return _babelTypes2.default.jSXAttribute(_babelTypes2.default.jSXIdentifier(name), jsxValue);
  }).filter(Boolean);
  if (classes.length) {
    var value = classes.every(function (cls) {
      return _babelTypes2.default.isStringLiteral(cls);
    }) ? _babelTypes2.default.stringLiteral(classes.map(function (cls) {
      return cls.value;
    }).join(' ')) : _babelTypes2.default.jSXExpressionContainer(_babelTypes2.default.callExpression(_babelTypes2.default.memberExpression(_babelTypes2.default.arrayExpression(classes), _babelTypes2.default.identifier('join')), [_babelTypes2.default.stringLiteral(' ')]));
    attrs.push(_babelTypes2.default.jSXAttribute(_babelTypes2.default.jSXIdentifier('className'), value));
  }
  return attrs;
}

function build(node, context) {
  var name = _babelTypes2.default.jSXIdentifier(node.name);
  var children = getChildren(node, context);
  if (node.attributeBlocks.length) {
    throw new Error('Attribute blocks are not yet supported in react-pug');
  }
  var attrs = getAttributes(node, context);
  context.key.handleAttributes(attrs);

  var open = _babelTypes2.default.jSXOpeningElement(name, attrs, // Array<JSXAttribute | JSXSpreadAttribute>
  children.length === 0);

  var close = children.length === 0 ? null : _babelTypes2.default.jSXClosingElement(name);

  return _babelTypes2.default.jSXElement(open, close, children, children.length === 0);
}

var TagVisitor = {
  jsx: function jsx(node, context) {
    return build(node, context);
  },
  expression: function expression(node, context) {
    return build(node, context);
  }
};

exports.default = TagVisitor;